/*
 * @(#)file      RequestNetworkData.java
 * @(#)author    Stephen Morris
 */

import java.io.*;
import java.util.Date;
import com.sun.management.snmp.SnmpDefinitions;
import com.sun.management.snmp.SnmpOid;
import com.sun.management.snmp.SnmpVarBindList;
import com.sun.management.snmp.manager.SnmpPeer;
import com.sun.management.snmp.manager.SnmpParameters;
import com.sun.management.snmp.manager.SnmpRequest;
import com.sun.management.snmp.manager.SnmpSession;

import com.sun.management.snmp.SnmpEventReportDispatcher;
import com.sun.management.snmp.SnmpOidTableSupport;

/**
 * The class is a simple example showing how to use the SNMP Manager API.
 *
 * <p>When calling the program, you must specify:
 * <br>     - host: hostname of the SNMP agent you want to query.
 * <br>     - port: port number to use - 161 is the standard
 *
 */

public class RequestNetworkData {

    /**
     * Main entry point.
     * <p>When calling the program, you must specify:
     * <br> 1) host: hostname of the SNMP agent you want to query.
     * <br> 2) port: port number to use - 161 is the standard
     *
     */

    public RequestNetworkData(String argv[]) {
		if (argv.length != 2) {
			usage();
			java.lang.System.exit(1);
		}
	}

    public void collectNetworkData(String argv[]) {

        final String host = argv[0];
        final String port = argv[1];
        final int WAIT_TIME = 10000;
        final int MS_DIVIDEND = 1000;

        try {
            // Initialize the SNMP Manager API.
            // Specify the OidTable containing all the MIB II information.
            // Use the OidTable generated by mibgen when compiling MIB II.
            //
            final SnmpOidTableSupport oidTable = new RFC1213_MIBOidTable();
            SnmpOid.setSnmpOidTable(oidTable);

            // Create a SnmpPeer object for representing the entity to
	    // communicate with.
            //
            final SnmpPeer agent =
		new SnmpPeer(host, Integer.parseInt(port));

            // Create parameters to associate to the entity to communicate
	    // with.
            // When creating the parameter object, you can specify the read
	    // and write community to be used when querying the agent.
            //
            final SnmpParameters params =
		new SnmpParameters("public", "private");

            // The newly created parameter must be associated to the agent.
            //
            agent.setParams(params);

            // Build the session. A session creates, controls and
	    // manages one or more requests.
            //
            final SnmpSession session =
		new SnmpSession("SyncManager session");

            // A default peer (agent) can be associated to a SnmpSession.
	    // When invoking a service provided by the SnmpSession, if the
	    // agent is not specified, the session will perform the service
	    // using the default peer as the target of the service.
            //
            session.setDefaultPeer(agent);

            // Build the list of variables you want to query.
            // For debug purposes, you can associate a name to your list.
            //
            final SnmpVarBindList list =
		new SnmpVarBindList("SyncManager varbind list");

            // We want to read a number of object instances.
			//
            list.addVarBind("sysDescr.0");
            list.addVarBind("ifOperStatus.1");
            list.addVarBind("ifInOctets.1");
            list.addVarBind("ifSpeed.1");

            // Make the SNMP get request and wait for the result.
            //
            // Date date1 = new Date();

            SnmpVarBindList vbList =
            	dispatchRequest(list, session, host, port, WAIT_TIME);
            println("Result: \n" + vbList);

            try {
			        BufferedWriter out = new BufferedWriter(new FileWriter("networkData.txt"));
			        out.write(vbList.varBindListToString(), 0, vbList.varBindListToString().length());
			        out.close();
			    } catch (IOException e) {
			    }


            // De-allocate the session resources
            //
            session.destroySession();

            //java.lang.System.exit(0);

        } catch (Exception e) {
            java.lang.System.err.println("RequestData::main: " +
					 "Exception occurred:" + e );
            e.printStackTrace();
        }
    }

    public static SnmpVarBindList dispatchRequest(SnmpVarBindList list,
    SnmpSession session, String host, String port, int timeout)
    throws Exception
    {
		// Make the SNMP get request and wait for the result.
        //
		SnmpRequest request = session.snmpGetRequest(null, list);
		println("SyncManager::dispatchRequest: " +
		"Sending getRequest to SNMP agent on " + host +
		" at port " + port);

		final boolean completed = request.waitForCompletion(timeout);

		// Check for a timeout of the request.
		//
		if (completed == false) {
			println("RequestData::main: getRequest timed out." +
		" Check reachability of agent");

			// Print request. Parse the various OIDs
			//
			println("getRequest: " + request.toString());
			java.lang.System.exit(0);
		}
        // Now we have a response. Check if the response contains
	    // an error.
        //
		final int errorStatus = request.getErrorStatus();
		if (errorStatus != SnmpDefinitions.snmpRspNoError) {
			println("Error status = " +
		SnmpRequest.snmpErrorToString(errorStatus));
			println("Error index = " + request.getErrorIndex());
			java.lang.System.exit(0);
		}
		return request.getResponseVarBindList();
	}

    /**
     * Return usage of the program.
     */
    public static void  usage() {
        println("java RequestData <host> <port>");
        println("where");
        println("\t-host: hostname or IP address of the SNMP agent" +
		" you want to query.");
        println("\t-port: port number to use.");
    }

    /**
     * print and println extensions
     */

    private final static void println(String msg) {
	java.lang.System.out.println(msg);
    }

    private /*final static*/ void print(String msg) {
	java.lang.System.out.print(msg);
    }
}
